import unittest
import xmlrunner
import oscillations
import math


float_comp = 7
xml_output = True



class TestUnits (unittest.TestCase):
	
	def test_energyGigaelectronvolts(self):
		self.assertAlmostEqual(oscillations.units.GeV, 1e+9 * oscillations.units.eV, places=float_comp)
	
	def test_energyMegaelectronvolts(self):
		self.assertAlmostEqual(oscillations.units.MeV, 1e+6 * oscillations.units.eV, places=float_comp)
		
	def test_energyKiloelectronvolts(self):
		self.assertAlmostEqual(oscillations.units.keV, 1e+3 * oscillations.units.eV, places=float_comp)
		
	def test_energyMillielectronvolts(self):
		self.assertAlmostEqual(oscillations.units.meV, 1e-3 * oscillations.units.eV, places=float_comp)
	
	def test_energyGigaelectronvoltsMegaelectronvolts(self):
		self.assertAlmostEqual(oscillations.units.GeV, 1e+3 * oscillations.units.MeV, places=float_comp)
	
	def test_distanceKilomtres(self):
		self.assertAlmostEqual(oscillations.units.km, 1e+3 * oscillations.units.m, places=float_comp)
	
	def test_distanceCentimetres(self):
		self.assertAlmostEqual(oscillations.units.cm, 1e-2 * oscillations.units.m, places=float_comp)
	
	def test_distanceMillimetres(self):
		self.assertAlmostEqual(oscillations.units.mm, 1e-3 * oscillations.units.m, places=float_comp)
	
	def test_distanceCentimtresMillimetres(self):
		self.assertAlmostEqual(oscillations.units.cm, 1e+1 * oscillations.units.mm, places=float_comp)
	
	def test_angleDegreesRadians(self):
		self.assertAlmostEqual(
			oscillations.units.degrees,
			(math.pi/180.0) * oscillations.units.radians,
			places=float_comp)
	
	def test_massMillelectronvolts2Electronvolts2(self):
		self.assertAlmostEqual(
			oscillations.units.meV2,
			( pow(1e-3, 2) * oscillations.units.eV2 ),
			places=float_comp)
	
	def test_distancePerEnergy(self):
		self.assertAlmostEqual(
			oscillations.units.km_GeV,
			oscillations.units.km / oscillations.units.GeV,
			places=float_comp)



class TestNeutrinos (unittest.TestCase):
	
	def test_nuEIsNeutrino(self):
		self.assertTrue( oscillations.isNeutrino( oscillations.nu_e ) )
	
	def test_nuMuIsNeutrino(self):
		self.assertTrue( oscillations.isNeutrino( oscillations.nu_mu ) )
	
	def test_nuTauIsNeutrino(self):
		self.assertTrue( oscillations.isNeutrino( oscillations.nu_tau ) )
	
	def test_nuEBarIsNotNeutrino(self):
		self.assertFalse( oscillations.isNeutrino( oscillations.nu_e_bar ) )
	
	def test_nuMuBarIsNotNeutrino(self):
		self.assertFalse( oscillations.isNeutrino( oscillations.nu_mu_bar ) )
	
	def test_nuTauBarIsNotNeutrino(self):
		self.assertFalse( oscillations.isNeutrino( oscillations.nu_tau_bar ) )
	
	def test_nuEIsNotAntiNeutrino(self):
		self.assertFalse( oscillations.isAntiNeutrino( oscillations.nu_e ) )
	
	def test_nuMuIsNotAntiNeutrino(self):
		self.assertFalse( oscillations.isAntiNeutrino( oscillations.nu_mu ) )
	
	def test_nuTauIsNotAntiNeutrino(self):
		self.assertFalse( oscillations.isAntiNeutrino( oscillations.nu_tau ) )
	
	def test_nuEBarIsAntiNeutrino(self):
		self.assertTrue( oscillations.isAntiNeutrino( oscillations.nu_e_bar ) )
	
	def test_nuMuBarIsAntiNeutrino(self):
		self.assertTrue( oscillations.isAntiNeutrino( oscillations.nu_mu_bar ) )
	
	def test_nuTauBarIsAntiNeutrino(self):
		self.assertTrue( oscillations.isAntiNeutrino( oscillations.nu_tau_bar ) )
	
	def test_nuEIsMinusNuEBar(self):
		self.assertEqual( oscillations.nu_e, -oscillations.nu_e_bar )
		
	def test_nuMuIsMinusNuMuBar(self):
		self.assertEqual( oscillations.nu_mu, -oscillations.nu_mu_bar )
		
	def test_nuTauIsMinusNuTauBar(self):
		self.assertEqual( oscillations.nu_tau, -oscillations.nu_tau_bar )
	
	def test_thereAreThreeNeutrinos(self):
		self.assertEqual( len(oscillations.neutrinos), 3 )
	
	def test_thereAreThreeAntiNeutrinos(self):
		self.assertEqual( len(oscillations.anti_neutrinos), 3 )
	
	def test_allStatesAreUnique(self):
		for neutrino in oscillations.neutrinos:
			for anti_neutrino in oscillations.anti_neutrinos:
				self.assertNotEqual(neutrino, anti_neutrino)



class TestOscillations (unittest.TestCase):
	
	def setUp(self):
		# Example experiment oscillations
		self.osc = oscillations.Oscillations()
		self.osc.setL(295.0 * oscillations.units.km )
		self.osc.setE(0.6   * oscillations.units.GeV)
		self.osc.setDeltaM21( 7.50e-5 * oscillations.units.eV2 )
		self.osc.setDeltaM32( 2.32e-3 * oscillations.units.eV2 )
		self.osc.setTheta12( 33.9 * oscillations.units.degrees )
		self.osc.setTheta13(  9.1 * oscillations.units.degrees )
		self.osc.setTheta23( 45.0 * oscillations.units.degrees )
		self.osc.setDeltaCP(  0.0 * oscillations.units.degrees )
		# get an index which does not represent a neutrino or anti-neutrino
		non_nu = 1
		while( oscillations.isNeutrino(non_nu) or oscillations.isAntiNeutrino(non_nu) ):
			non_nu += 1
		self.non_neutrino = non_nu
	
	def test_setERaisesValueErrorForNonFloat(self):
		self.assertRaises(ValueError, self.osc.setE, "a")
	
	def test_setERaisesValueErrorForNegativeValue(self):
		self.assertRaises(ValueError, self.osc.setE, -3.3)
	
	def test_setLRaisesValueErrorForNonFloat(self):
		self.assertRaises(ValueError, self.osc.setL, "b")
	
	def test_setLRaisesValueErrorForNegativeValue(self):
		self.assertRaises(ValueError, self.osc.setL, -2.4)
	
	def test_setLOverERaisesValueErrorForNonFloat(self):
		self.assertRaises(ValueError, self.osc.setLOverE, "c")
	
	def test_setLOverERaisesValueErrorForNegativeValue(self):
		self.assertRaises(ValueError, self.osc.setLOverE, -8.1)
	
	def test_setLOverESucceedsInSettingValue(self):
		new_l_over_e = 1.3
		self.osc.setLOverE(new_l_over_e)
		self.assertAlmostEqual( self.osc.lOverE(), new_l_over_e, places=float_comp )
	
	def test_setDeltaM32RaisesValueErrorForNonFloat(self):
		self.assertRaises(ValueError, self.osc.setDeltaM32, "d")
	
	def test_setDeltaM21RaisesValueErrorForNonFloat(self):
		self.assertRaises(ValueError, self.osc.setDeltaM21, "e")
	
	def test_setTheta12RaisesValueErrorForNonFloat(self):
		self.assertRaises(ValueError, self.osc.setTheta12, "f")
	
	def test_setTheta23RaisesValueErrorForNonFloat(self):
		self.assertRaises(ValueError, self.osc.setTheta23, "g")
	
	def test_setTheta13RaisesValueErrorForNonFloat(self):
		self.assertRaises(ValueError, self.osc.setTheta13, "h")
	
	def test_setDeltaCPRaisesValueErrorForNonFloat(self):
		self.assertRaises(ValueError, self.osc.setDeltaCP, "i")
	
	def test_oscillationsOccur(self):
		for initial in oscillations.neutrinos:
			for final in oscillations.neutrinos:
				p = self.osc.p(initial, final)
				self.assertGreater( p , 0.0 )
				self.assertLessEqual( p, 1.0 )
	
	def test_unitarity(self):
		total = 0.0
		for neutrino in oscillations.neutrinos:
			total += self.osc.p(oscillations.nu_mu, neutrino)
		self.assertAlmostEqual( total, 1.0, places=float_comp )
	
	def test_noCPViolationIfDeltaIsZero(self):
		self.osc.setDeltaCP(0.0)
		self.assertAlmostEqual(
			self.osc.p(oscillations.nu_mu, oscillations.nu_e),
			self.osc.p(oscillations.nu_mu_bar, oscillations.nu_e_bar),
			places=float_comp)
	
	def test_noCPViolationIfTheta13IsZero(self):
		self.osc.setTheta13(0.0)
		self.assertAlmostEqual(
			self.osc.p(oscillations.nu_mu, oscillations.nu_e),
			self.osc.p(oscillations.nu_mu_bar, oscillations.nu_e_bar),
			places=float_comp)
	
	def test_cpViolationIfDeltaNonZero(self):
		self.osc.setTheta13(9.1  * oscillations.units.degrees)
		self.osc.setDeltaCP(35.0 * oscillations.units.degrees)
		self.assertNotAlmostEqual(
			self.osc.p(oscillations.nu_mu, oscillations.nu_e),
			self.osc.p(oscillations.nu_mu_bar, oscillations.nu_e_bar),
			places=float_comp)
	
	def test_noOscillationsAtDistanceZero(self):
		self.osc.setL(0.0)
		self.assertEqual( self.osc.p(oscillations.nu_e, oscillations.nu_e  ), 1.0)
		self.assertEqual( self.osc.p(oscillations.nu_e, oscillations.nu_mu ), 0.0)
		self.assertEqual( self.osc.p(oscillations.nu_e, oscillations.nu_tau), 0.0)
	
	def test_noOscillationsAtEnergyZero(self):
		self.osc.setE(0.0)
		self.assertEqual( self.osc.p(oscillations.nu_e, oscillations.nu_e  ), 1.0)
		self.assertEqual( self.osc.p(oscillations.nu_e, oscillations.nu_mu ), 0.0)
		self.assertEqual( self.osc.p(oscillations.nu_e, oscillations.nu_tau), 0.0)
	
	def test_noOscillationsBetweenNeutrinosAndAntiNeutrions(self):
		for neutrino in oscillations.neutrinos:
			for anti_neutrino in oscillations.anti_neutrinos:
				self.assertEqual( self.osc.p(neutrino, anti_neutrino), 0.0 )
				self.assertEqual( self.osc.p(anti_neutrino, neutrino), 0.0 )
	
	def test_noOscillationsIfNoMassDifferences(self):
		self.osc.setDeltaM21(0.0)
		self.osc.setDeltaM32(0.0)
		self.assertAlmostEqual( self.osc.p(oscillations.nu_mu,oscillations.nu_e  ), 0.0, places=float_comp )
		self.assertAlmostEqual( self.osc.p(oscillations.nu_mu,oscillations.nu_mu ), 1.0, places=float_comp )
		self.assertAlmostEqual( self.osc.p(oscillations.nu_mu,oscillations.nu_tau), 0.0, places=float_comp )
	
	def test_noOscillationsIfNoMixing(self):
		self.osc.setTheta12(0.0)
		self.osc.setTheta23(0.0)
		self.osc.setTheta13(0.0)
		self.assertAlmostEqual( self.osc.p(oscillations.nu_mu,oscillations.nu_e  ), 0.0, places=float_comp )
		self.assertAlmostEqual( self.osc.p(oscillations.nu_mu,oscillations.nu_mu ), 1.0, places=float_comp )
		self.assertAlmostEqual( self.osc.p(oscillations.nu_mu,oscillations.nu_tau), 0.0, places=float_comp )
	
	def test_pRaisesValueErrorForInvalidInitialState(self):
		self.assertRaises(ValueError, self.osc.p, self.non_neutrino, oscillations.nu_mu)
	
	def test_pRaisesValueErrorForInvalidFinalState(self):
		self.assertRaises(ValueError, self.osc.p, oscillations.nu_mu, self.non_neutrino)



def main():
	if( xml_output ):
		import xmlrunner	
		unittest.main(
			testRunner=xmlrunner.XMLTestRunner(),
			failfast=False,
			buffer=False,
			catchbreak=False
			)
	else:
		unittest.main()
	
if __name__ == "__main__":
	main()



from setuptools import setup

setup(
	name="oscillations",
	version="1.1.1",
	description="Calculate neutrino oscillation probabilities easily",
	long_description="Calculate neutrino oscillation probabilities easily",
	url="https://github.com/discully/oscillations",
	author="Daniel I. Scully",
	author_email="discully@users.noreply.github.com",
	packages=["oscillations"],
	install_requires=[],
	classifiers=["Programming Language :: Python :: 3"]
)



"""
Module for Calculating neutrino oscillation probabilities easily.

class Oscillations is the primary interface, which allows you to set the
various parameters and calculate oscillation probabilities.

The 'units' object provides pre-defined constants to ensure parameters
provided to Oscillations are in the correct units.

An enumeration of the 6 neutrino/anti-neutrino flavours is provided in
the form of nu_e, nu_mu, nu_tau, nu_e_bar, nu_mu_bar and nu_tau_bar.

For convenience, groups of those enumerations (neutrinos and anti_neutrinos)
are provided, as are some functions (isNeutrino() and isAntiNeutrino())for
testing them.
"""


__version__ = "1.1.1"
__author__ = "Daniel I. Scully"


import math
import cmath



#
# Units
#



class Units:
	"""Various factors for keeping units of physical quantities internally consistent.

	Here's how to set in a value in a given units:
	one_kilometer = 1.0 * units.km

	Here's how to get a value in a given units:
	print "1km = ", one_kilometer / units.mm, "mm"
	"""

	def __init__(self):
		# Energy
		self.GeV = 1.0
		self.eV  = 1.0e-09 * self.GeV
		self.meV = 1.0e-03 * self.eV
		self.keV = 1.0e+03 * self.eV
		self.MeV = 1.0e+06 * self.eV
		self.TeV = 1.0e+12 * self.eV
		# Distance
		self.m  = 5.07e+15 / self.GeV
		self.km = 1.0e+03 * self.m
		self.mm = 1.0e-03 * self.m
		self.cm = 1.0e-02 * self.m
		# Angle
		self.radians = 1.0
		self.degrees = (math.pi/180.0) * self.radians
		# For mass-squared differences
		self.eV2  = pow(self.eV,  2)
		self.meV2 = pow(self.meV, 2)
		# For L/E
		self.km_GeV = self.km / self.GeV
units = Units()



#
# Neutrino / Anti-Neutrino definitions
#



# Enumeration of possible neutrino/anti-neutrino states.
# Should be used when calling Oscillations::p().
nu_e = 1
nu_mu = 2
nu_tau = 3
nu_e_bar = -nu_e
nu_mu_bar = -nu_mu
nu_tau_bar = -nu_tau


# Convenience collections of the neutrino/anti-neutrino states.
neutrinos = (nu_e, nu_mu, nu_tau)
anti_neutrinos = (nu_e_bar, nu_mu_bar, nu_tau_bar)


def isNeutrino(state):
	"""Returns True if state is a valid neutrino enum, or False otherwise."""
	return (state in neutrinos)


def isAntiNeutrino(state):
	"""Returns True if state is a valid anti-neutrino enum, or False otherwise."""
	return (state in anti_neutrinos)



#
# Oscillation calculations
#



class Oscillations:
	"""Making PMNS neutrino oscillation calculations.

	Methods are provided to set the PMNS parameters, mass-squared differences,
	and experimental energy and baseline.
	Oscillation probabilities can than be calculated for given initial
	and final neutrino states.

	Matter effects are not currently supported.
	"""

	def __init__(self):
		"""Constructs with initial parameters from my thesis and the T2K experiment."""
		self.L = 295.0 * units.km  # T2K approximate baseline
		self.E =   0.6 * units.GeV # T2K approximate peak nu_mu energy

		self.delta_m2_21 = (7.50e-5) * units.eV2
		self.delta_m2_32 = (2.32e-3) * units.eV2
		self._updateMasses()

		self.theta_12 = 33.9 * units.degrees
		self.theta_13 =  9.1 * units.degrees
		self.theta_23 = 45.0 * units.degrees
		self.delta_cp =  0.0 * units.degrees
		self._updateMatrix()


	def lOverE(self):
		"""Return the value of L/E."""
		if( self.E > 0.0 ):
			return (self.L / self.E)
		else:
			return 0.0


	def setE(self, energy):
		"""Set the neutrino energy.

		Raises ValueError if energy cannot be converted to a float.
		Raises ValueError if energy is negative.
		"""
		energy = float(energy) * units.GeV
		if( energy < 0.0 ):
			raise ValueError("Neutrino energy must be positive.")
		self.E = energy


	def setL(self, baseline):
		"""Set the oscillation baseline.

		Raises ValueError if baseline cannot be converted to a float.
		Raises ValueError if baseline is negative.
		"""
		baseline = float(baseline)
		if( baseline < 0.0 ):
			raise ValueError("Oscillation baseline must be positive.")
		self.L = baseline


	def setLOverE(self, l_over_e):
		"""Overrides the current L and E to set the ratio L/E.

		No guarantees are made about what L or E will be set to in order to achieve this.
		Raises ValueError if l_over_e cannot be converted to a float.
		Raises ValueError if l_over_e is negative.
		"""
		l_over_e = float(l_over_e)
		if( l_over_e < 0.0 ):
			raise ValueError("L/E must be positive.")
		self.L = self.E * l_over_e


	def setDeltaM32(self, dm2):
		"""Set the neutrino mass-squared difference (Delta m^2)_32.

		(Delta m^2)_32 = (m_3)^2 - (m_2)^2
		Raises ValueError if dm2 cannot be converted to a float.
		"""
		self.delta_m2_32 = float(dm2)
		self._updateMasses()


	def setDeltaM21(self, dm2):
		"""Set the neutrino mass-squared difference (Delta m^2)_21.

		(Delta m^2)_21 = (m_2)^2 - (m_1)^2
		Raises ValueError if dm2 cannot be converted to a float.
		"""
		self.delta_m2_21 = float(dm2)
		self._updateMasses()


	def setTheta12(self, theta):
		"""Set the PMNS mixing angle theta_12.

		Raises ValueError is theta_radians cannot be converted to a float.
		"""
		self.theta_12 = float(theta)
		self._updateMatrix()


	def setTheta23(self, theta):
		"""Set the PMNS mixing angle theta_23.

		Raises ValueError is theta cannot be converted to a float.
		"""
		self.theta_23 = float(theta)
		self._updateMatrix()


	def setTheta13(self, theta):
		"""Set the PMNS mixing angle theta_13.

		Raises ValueError is theta cannot be converted to a float.
		"""
		self.theta_13 = float(theta)
		self._updateMatrix()


	def setDeltaCP(self, delta):
		"""Set the PMNS CP-violating phase delta_cp.

		Raises ValueError is delta cannot be converted to a float.
		"""
		self.delta_cp = float(delta)
		self._updateMatrix()


	def _updateMatrix(self):
		"""Updates the PMNS matrix and its complex conjugate.

		Must be called by the class each time one of the PMNS matrix parameters are changed.
		"""
		zero = complex( 0.0, 0.0 )
		c12  = complex( math.cos( self.theta_12 ), 0.0 )
		c13  = complex( math.cos( self.theta_13 ), 0.0 )
		c23  = complex( math.cos( self.theta_23 ), 0.0 )
		s12  = complex( math.sin( self.theta_12 ), 0.0 )
		s13  = complex( math.sin( self.theta_13 ), 0.0 )
		s23  = complex( math.sin( self.theta_23 ), 0.0 )
		eid  = cmath.exp( complex(0.0,  self.delta_cp) ) # e^( i * delta_cp)
		emid = cmath.exp( complex(0.0, -self.delta_cp) ) # e^(-i * delta_cp)

		self.matrix      = [[zero,zero,zero],[zero,zero,zero],[zero,zero,zero]]
		self.anti_matrix = [[zero,zero,zero],[zero,zero,zero],[zero,zero,zero]]

		self.matrix[0][0] = c12 * c13
		self.matrix[0][1] = s12 * c13
		self.matrix[0][2] = s13 * emid

		self.matrix[1][0] = (zero - s12*c23 ) - ( c12*s23*s13*eid )
		self.matrix[1][1] = ( c12*c23 ) - ( s12*s23*s13*eid )
		self.matrix[1][2] = s23*c13

		self.matrix[2][0] = ( s12*s23 ) - ( c12*c23*s13*eid)
		self.matrix[2][1] = ( zero - c12*s23 ) - ( s12*c23*s13*eid )
		self.matrix[2][2] = c23*c13

		for i in range(3):
			for j in range(3):
				self.anti_matrix[i][j] = self.matrix[i][j].conjugate()


	def _updateMasses(self):
		"""Updates the neutrino masses (squared).

		Must be called by the class each time one of the mass-squared differences are changed.
		"""
		# Remember oscillations are insensitive to the absolute scale of the masses.
		# Here, we assume the smallest mass is 0.
		m2_2 = max(self.delta_m2_21, self.delta_m2_32)
		m1_2 = m2_2 - self.delta_m2_21
		m3_2 = m2_2 + self.delta_m2_32
		self.mass_squared = [m1_2, m2_2, m3_2]


	def p(self, initial, final):
		"""Returns the oscillation probability.

		initial : The initial state neutrino
		final : The final state neutrino

		Raises ValueError if initial/final is not neutrino/anti-neutrino enum as defined in this module.
		If initial and final are not both neutrinos/anti-neutrinos, 1.0 is returned.
		If L or E is 0.0, then 0.0 is returned, or 1.0 if initial == final.
		"""

		if( not isNeutrino(initial) and not isAntiNeutrino(initial) ):
			raise ValueError("Invalid value for initial neutrino state.")
		elif( not isNeutrino(final) and not isAntiNeutrino(final) ):
			raise ValueError("Invalid value for final neutrino state.")

		if( isNeutrino(initial) and isAntiNeutrino(final) ):
			return 0.0 # probability of nu -> anti_nu oscillation
		elif( isAntiNeutrino(initial) and isNeutrino(final) ):
			return 0.0 # probability of anti_nu ->nu oscillation

		L = self.L
		E = self.E
		# Oscillations with E = 0 or don't really make sense,
		# but if you're plotting graphs these are the values you'll want.
		if( E == 0.0 or L == 0.0 ):
			if( initial == final ):
				return 1.0
			else:
				return 0.0

		if( isNeutrino(initial) ):
			U = self.matrix      # Use PMNS matrix
		else:
			U = self.anti_matrix # Use complex-conjugate of PMNS matrix

		a = abs(initial) - 1 # index of initial neutrino state
		b = abs(final)   - 1 # index of final neutrino state
		m2 = self.mass_squared
		i = complex(0,1)

		s = complex(0.0,0.0)
		for x in range(3):
			s += U[a][x].conjugate() * U[b][x] * cmath.exp( (-i*m2[x]*L)/(2.0*E) )

		return pow(abs(s), 2)


	def __str__(self):
		s  = "theta_12 = {:.2f} degrees\n".format(self.theta_12/units.degrees)
		s += "theta_23 = {:.2f} degrees\n".format(self.theta_23/units.degrees)
		s += "theta_13 = {:.2f} degrees\n".format(self.theta_13/units.degrees)
		m2 = self.mass_squared
		s += "(Delta m^2)_21 = {:.2f} meV^2\n".format( (m2[1] - m2[0] )/units.meV2 )
		s += "(Delta m^2)_32 = {:.2f} meV^2\n".format( (m2[2] - m2[1] )/units.meV2 )
		s += "(Delta m^2)_31 = {:.2f} meV^2\n".format( (m2[2] - m2[0] )/units.meV2 )
		s += "L   = {:} km \n".format(self.L/units.km )
		s += "E   = {:} GeV\n".format(self.E/units.GeV)
		s += "L/E = {:.2f} km/GeV".format((self.L/units.km) / (self.E/units.GeV))
		return s



#
# Preset oscillation parameter sets
#



def pdg2013():
	"""Returns an Oscillations instance with parameters set to the PDG 2013 values."""
	osc = Oscillations()
	theta_12 = math.asin(math.sqrt(0.857)) / 2.0
	osc.setTheta12(theta_12 * units.radians)
	osc.setDeltaM21(7.50e-5 * units.eV2)
	osc.setTheta23(45.0 * units.degrees)
	osc.setDeltaM32(2.32e-3 * units.eV2)
	theta_13 = math.asin(math.sqrt(0.095)) / 2.0
	osc.setTheta13(theta_13 * units.radians)
	osc.setDeltaCP(0.0)
	return osc


def pdg2020():
	"""Returns an Oscillations instance with parameters set to the PDG 2020 (normal order) values."""
	osc = Oscillations()
	theta_12 = math.asin(math.sqrt(0.307))
	osc.setTheta12(theta_12 * units.radians)
	osc.setDeltaM21(7.53e-5 * units.eV2)
	theta_23 = math.asin(math.sqrt(0.545))
	osc.setTheta23(theta_23 * units.radians)
	osc.setDeltaM32(2.453e-3 * units.eV2)
	theta_13 = math.asin(math.sqrt(2.18e-2))
	osc.setTheta13(theta_13 * units.radians)
	osc.setDeltaCP(1.36 * math.pi * units.radians)
	return osc


def pdg2022():
	"""Returns an Oscillations instance with parameters set to the PDG 2022 (normal order) values."""
	osc = Oscillations()
	theta_12 = math.asin(math.sqrt(0.307))
	osc.setTheta12(theta_12 * units.radians)
	osc.setDeltaM21(7.53e-5 * units.eV2)
	theta_23 = math.asin(math.sqrt(0.546))
	osc.setDeltaM32(2.453e-3 * units.eV2)
	theta_13 = math.asin(math.sqrt(2.20e-2))
	osc.setTheta13(theta_13 * units.radians)
	osc.setDeltaCP(1.36 * math.pi * units.radians)
	return osc

